library(readxl) # import excel files (if needed)
library(dplyr) # data manipulation
library(psych) # alphas, descriptive statistics, ...
library(caret) # can be used for different machine learning techniques; here used for splitting into training and test data set and random forest analyses
library(sjPlot) # mean inter-item correlations of big five domains and facets
library(ranger) # to use the "ranger" method (from the ranger package) alongside the caret package (implements random forest analyses)


#####################################################################
####### Load Data Set (participants left after data cleaning) #######
#####################################################################

### EITHER import excel file ###
data_bf_partypref_clean <- read_excel("/.../datafile_sample_participants_rf_bf_voting.xlsx", sheet = "data", col_types = "numeric") # read data into R
codebook_data_bf_partypref <- read_excel("/.../datafile_sample_participants_rf_bf_voting.xlsx", sheet = "codebook") # read code book into R

### OR import RData file ###
#load("/.../data_and_codebook.RData")

#note: in this code the term "party preference" rather than "voting intention" is used.

#########################################################
####### First Analysis of Sample Characteristics  #######
#########################################################

#### sample characteristics ####
table(data_bf_partypref_clean$Sex); prop.table(table(data_bf_partypref_clean$Sex)) # tables on how many (absolute and in percentage (if multiplied with 100)) individuals were men / women or indicated "third gender" as gender identity
psych::describe(data_bf_partypref_clean$Age) # descriptive statistics on age in the sample
table(data_bf_partypref_clean$Education) # table on how many individuals indicated which degree as their highest educational degree (not mentioned in the manuscript)
table(data_bf_partypref_clean$partypref); prop.table(table(data_bf_partypref_clean$partypref))  # distribution of voting intentions plus table on voting intentions in percentages (if multiplied with 100)


#########################################################
####### Calculate BFI (Big Five Inventory) Scores #######
#########################################################

#### Recoding Items ####

data_bf_partypref_clean  <- data_bf_partypref_clean  %>%
        dplyr::mutate(BFI_06_R = reverse.code(-1, BFI_06, 1, 5),
                      BFI_21_R = reverse.code(-1, BFI_21, 1, 5),
                      BFI_31_R = reverse.code(-1, BFI_31, 1, 5),
                      BFI_02_R = reverse.code(-1, BFI_02, 1, 5),
                      BFI_12_R = reverse.code(-1, BFI_12, 1, 5),
                      BFI_27_R = reverse.code(-1, BFI_27, 1, 5),
                      BFI_37_R = reverse.code(-1, BFI_37, 1, 5),
                      BFI_08_R = reverse.code(-1, BFI_08, 1, 5),
                      BFI_18_R = reverse.code(-1, BFI_18, 1, 5),
                      BFI_23_R = reverse.code(-1, BFI_23, 1, 5),
                      BFI_43_R = reverse.code(-1, BFI_43, 1, 5),
                      BFI_09_R = reverse.code(-1, BFI_09, 1, 5),
                      BFI_24_R = reverse.code(-1, BFI_24, 1, 5),
                      BFI_34_R = reverse.code(-1, BFI_34, 1, 5),
                      BFI_35_R = reverse.code(-1, BFI_35, 1, 5),
                      BFI_41_R = reverse.code(-1, BFI_41, 1, 5))

#### Big Five Domains ####

data_bf_partypref_clean$open <- data_bf_partypref_clean %>%
        dplyr::select("BFI_05", "BFI_10", "BFI_15", "BFI_20", "BFI_25", "BFI_30", "BFI_35_R", "BFI_40","BFI_41_R", "BFI_44") %>%
        rowMeans() # calculates mean across items and creates new variable (here: open) in dataset
data_bf_partypref_clean %>%
        dplyr::select("BFI_05", "BFI_10", "BFI_15", "BFI_20", "BFI_25", "BFI_30", "BFI_35_R", "BFI_40","BFI_41_R", "BFI_44") %>%
        psych::alpha() # reliability analysis of items / scale; among others: Cronbach's alpha
data_bf_partypref_clean %>%
        dplyr::select("BFI_05", "BFI_10", "BFI_15", "BFI_20", "BFI_25", "BFI_30", "BFI_35_R", "BFI_40","BFI_41_R", "BFI_44") %>%
        tab_itemscale() # among others inter-item-correlations as output

data_bf_partypref_clean$consc <- data_bf_partypref_clean %>%
        dplyr::select("BFI_03", "BFI_08_R", "BFI_13", "BFI_18_R", "BFI_23_R", "BFI_28", "BFI_33", "BFI_38","BFI_43_R") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_03", "BFI_08_R", "BFI_13", "BFI_18_R", "BFI_23_R", "BFI_28", "BFI_33", "BFI_38","BFI_43_R") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_03", "BFI_08_R", "BFI_13", "BFI_18_R", "BFI_23_R", "BFI_28", "BFI_33", "BFI_38","BFI_43_R") %>%
        tab_itemscale()

data_bf_partypref_clean$extra <- data_bf_partypref_clean %>%
        dplyr::select("BFI_01", "BFI_06_R", "BFI_11", "BFI_16", "BFI_21_R", "BFI_26", "BFI_31_R", "BFI_36") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_01", "BFI_06_R", "BFI_11", "BFI_16", "BFI_21_R", "BFI_26", "BFI_31_R", "BFI_36") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_01", "BFI_06_R", "BFI_11", "BFI_16", "BFI_21_R", "BFI_26", "BFI_31_R", "BFI_36") %>%
        tab_itemscale()

data_bf_partypref_clean$agree <- data_bf_partypref_clean %>%
        dplyr::select("BFI_02_R", "BFI_07", "BFI_12_R", "BFI_17", "BFI_22", "BFI_27_R", "BFI_32", "BFI_37_R","BFI_42") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_02_R", "BFI_07", "BFI_12_R", "BFI_17", "BFI_22", "BFI_27_R", "BFI_32", "BFI_37_R","BFI_42") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_02_R", "BFI_07", "BFI_12_R", "BFI_17", "BFI_22", "BFI_27_R", "BFI_32", "BFI_37_R","BFI_42") %>%
        tab_itemscale()

data_bf_partypref_clean$neuro <- data_bf_partypref_clean %>%
        dplyr::select("BFI_04", "BFI_09_R", "BFI_14", "BFI_19", "BFI_24_R", "BFI_29", "BFI_34_R", "BFI_39") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_04", "BFI_09_R", "BFI_14", "BFI_19", "BFI_24_R", "BFI_29", "BFI_34_R", "BFI_39") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_04", "BFI_09_R", "BFI_14", "BFI_19", "BFI_24_R", "BFI_29", "BFI_34_R", "BFI_39") %>%
        tab_itemscale()

#### Big Five Facets ####

data_bf_partypref_clean$open_aes <- data_bf_partypref_clean %>%
        dplyr::select("BFI_30", "BFI_41_R", "BFI_44") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_30", "BFI_41_R", "BFI_44") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_30", "BFI_41_R", "BFI_44") %>%
        tab_itemscale()


data_bf_partypref_clean$open_id <- data_bf_partypref_clean %>%
        dplyr::select("BFI_10", "BFI_15", "BFI_25", "BFI_35_R", "BFI_40") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_10", "BFI_15", "BFI_25", "BFI_35_R", "BFI_40") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_10", "BFI_15", "BFI_25", "BFI_35_R", "BFI_40") %>%
        tab_itemscale()

data_bf_partypref_clean$consc_ord <- data_bf_partypref_clean %>%
        dplyr::select("BFI_08_R", "BFI_18_R") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_08_R", "BFI_18_R") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_08_R", "BFI_18_R") %>%
        tab_itemscale()

data_bf_partypref_clean$consc_sd <- data_bf_partypref_clean %>%
        dplyr::select("BFI_13", "BFI_23_R", "BFI_28", "BFI_38","BFI_43_R") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_13", "BFI_23_R", "BFI_28", "BFI_38","BFI_43_R") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_13", "BFI_23_R", "BFI_28", "BFI_38","BFI_43_R") %>%
        tab_itemscale()

data_bf_partypref_clean$extra_ass <- data_bf_partypref_clean %>%
        dplyr::select("BFI_01", "BFI_06_R", "BFI_21_R", "BFI_26", "BFI_31_R") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_01", "BFI_06_R", "BFI_21_R", "BFI_26", "BFI_31_R") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_01", "BFI_06_R", "BFI_21_R", "BFI_26", "BFI_31_R") %>%
        tab_itemscale()

data_bf_partypref_clean$extra_act <- data_bf_partypref_clean %>%
        dplyr::select("BFI_11", "BFI_16") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_11", "BFI_16") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_11", "BFI_16") %>%
        tab_itemscale()

data_bf_partypref_clean$agree_altr <- data_bf_partypref_clean %>%
        dplyr::select("BFI_07", "BFI_22", "BFI_27_R", "BFI_32") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_07", "BFI_22", "BFI_27_R", "BFI_32") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_07", "BFI_22", "BFI_27_R", "BFI_32") %>%
        tab_itemscale()

data_bf_partypref_clean$agree_com <- data_bf_partypref_clean %>%
        dplyr::select("BFI_02_R", "BFI_12_R", "BFI_17") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_02_R", "BFI_12_R", "BFI_17") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_02_R", "BFI_12_R", "BFI_17") %>%
        tab_itemscale()

data_bf_partypref_clean$neuro_anx <- data_bf_partypref_clean %>%
        dplyr::select("BFI_09_R", "BFI_19", "BFI_34_R", "BFI_39") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_09_R", "BFI_19", "BFI_34_R", "BFI_39") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_09_R", "BFI_19", "BFI_34_R", "BFI_39") %>%
        tab_itemscale()

data_bf_partypref_clean$neuro_dep <- data_bf_partypref_clean %>%
        dplyr::select("BFI_04", "BFI_29") %>%
        rowMeans()
data_bf_partypref_clean %>%
        dplyr::select("BFI_04", "BFI_29") %>%
        psych::alpha()
data_bf_partypref_clean %>%
        dplyr::select("BFI_04", "BFI_29") %>%
        tab_itemscale()


#########################################################
#### Descriptive Statistics of Big Five Personality #####
#### domains, facets, and nuances (recoded) #############
#########################################################

psych::describe(data_bf_partypref_clean[c("open", "consc", "extra", "agree", "neuro", 
                                          "open_aes", "open_id", "consc_ord", "consc_sd", "extra_ass", "extra_act", "agree_altr", "agree_com", "neuro_anx", "neuro_dep", 
                                          "BFI_01", "BFI_02_R", "BFI_03", "BFI_04", "BFI_05", "BFI_06_R", "BFI_07", "BFI_08_R", "BFI_09_R", "BFI_10",
                                          "BFI_11", "BFI_12_R", "BFI_13", "BFI_14", "BFI_15", "BFI_16", "BFI_17", "BFI_18_R", "BFI_19", "BFI_20",
                                          "BFI_21_R", "BFI_22", "BFI_23_R", "BFI_24_R", "BFI_25", "BFI_26", "BFI_27_R", "BFI_28", "BFI_29", "BFI_30",
                                          "BFI_31_R", "BFI_32", "BFI_33", "BFI_34_R", "BFI_35_R", "BFI_36", "BFI_37_R", "BFI_38", "BFI_39", "BFI_40",
                                          "BFI_41_R", "BFI_42", "BFI_43_R", "BFI_44")])
# note: descriptive statistics are presented in the Supplementary Material


################################################################
#### Pre-processing of Variables for Random Forest Analyses ####
#### (e.g. dependent variables as factors to make sure #########
#### algorithm "recognizes" the classification problem) ########
################################################################

data_bf_partypref_clean <- transform(data_bf_partypref_clean,
                                     partypref=as.factor(partypref),
                                     partypref2=as.factor(partypref2),
                                     partypref2leftright=as.factor(partypref2leftright),
                                     voter=as.factor(voter)) # all criterion variables in subsequent analyses are now factors --> classification problem will be recognized

levels(data_bf_partypref_clean$voter) = c("x0", "x1") # to calculate probabilities with ranger (see below), non-numeric factor levels are needed; therefore we rename factor levels to x0 and x1 instead of 0 and 1
levels(data_bf_partypref_clean$partypref2leftright) = c("x0", "x1")  # to calculate probabilities with ranger (see below), non-numeric factor levels are needed; therefore we rename factor levels to x0 and x1 instead of 0 and 1

data_bf_partypref_clean$partypref2 <- dplyr::recode(data_bf_partypref_clean$partypref2, "1"="5", "2"="2", "3"="3", "4"="4", "5"="1", "6"="6", "7"="7") # order parties from most left (1) to most right (6) party (as presented in main manuscript)
levels(data_bf_partypref_clean$partypref2) = list("x1" = "1", "x2" = "2", "x3" = "3", "x4" = "4", "x5" = "5", "x6" = "6", "x7" = "7") # to calculate probabilities with ranger (see below), non-numeric factor levels are needed; therefore we rename factor levels to x1 to x7 instead of 1 to 7


final_data <- data_bf_partypref_clean
rm(data_bf_partypref_clean)

################################################
####### Implement Random Forest Analyses ####### 
################################################
seed = 1234 #set seed = 1234 to ensure that always the same seed (1234) is used
RNGkind(sample.kind ="Rejection") # we set this in order to allow for replication of results across R-versions


################################################
######## Predict non-voting vs. voting ######### 
################################################

#### The folds need to be the same across all models in 10 x 10 CV: ####
set.seed(seed)
multifolds_voter <- createMultiFolds(final_data$voter, k = 10, times = 10)
#creates 10x10 folds with similar distribution of voter variable in all folds

#### trainControl is similar across analyses on prediction of non-voting vs. voting intentions: ####
trcontr_w1 <- trainControl(method = "repeatedcv", number = 10, repeats = 10, returnData = T, returnResamp = "all", savePredictions = "all", classProbs = T, index = multifolds_voter) # 10-fold cross validation and class probabilities and predictions should be saved, multifolds (see above are used)

##### writing the function #####
table(final_data$voter) # to calculate weights (see main manuscript for formula)
prop.table(table(final_data$voter))

function_voter1 = function(data){ # write one function to combine all necessary steps for the analyses. You can use the same function to predict non-voting vs voting by Big Five personality domains, facets, and items
        grid = expand.grid(mtry = round(sqrt(ncol(data)-1)), splitrule = "gini", min.node.size = 1) # set variables sampled at each split to square root of number of predictor variables, splitting rule to "gini", and minimal node size to 1
        weights_voter <- ifelse(data$voter == "x0", 4041/245, 1) # set weights
        set.seed(seed) 
        model <- train(voter ~ ., data = data, # model training (predict "voter" from other variables in data set specified as "data")
                       method = "ranger", # implements random forest analyses using the ranger method/package
                       weights = weights_voter, # set weights (see above)
                       trControl = trcontr_w1, # set trainControl (see above)
                       metric = "Kappa", # Kappa is used because it is better than accuracy for unbalanced prediction problems; but not needed here because all tuning parameters are set
                       tuneGrid = grid, # set tuneGrid (see above)
                       importance = "permutation") # calculate permutation importance
        print("model")
        model %>% print()
        print("cross-validation results")
        model$results %>% print()
        model$resample %>% print()
        print("final model")
        model$finalModel %>% print ()
        print("confusion Matrix across repeated CV")
        matrix <- confusionMatrix.train(model, norm = "none")$table %>% print()
        print("Classification errors")
        ((matrix[2,1]/(matrix[1,1] + matrix[2,1]))*100) %>% print()
        ((matrix[1,2]/(matrix[1,2] + matrix[2,2]))*100) %>% print()
        print("sensitivity = recall")
        rec <- (matrix[1,1]/(matrix[1,1] + matrix[2,1])) %>% print()
        print("specificity")
        (matrix[2,2]/(matrix[1,2] + matrix[2,2])) %>% print()
        print("precision")
        prec <- (matrix[1,1]/(matrix[1,1] + matrix[1,2])) %>% print()
        print("F1 score")
        F1_1 <- ((2*(prec*rec)) / (prec+rec)) %>% print()
        F1_2 <- (2 / ((1/rec)+(1/prec))) %>% print()
        F1_3 <- (matrix[1,1] / (matrix[1,1] + 0.5*(matrix[2,1] + matrix[1,2]))) %>% print() # different formulas are used in order to make sure no error occurs
        print("accuracy of model compared to no information rate")
        (((((matrix[1,1] + matrix[2,2]) / (matrix[1,1] + matrix[1,2] + matrix[2,1] + matrix[2,2])) - (as.matrix(prop.table(table(data$voter)))[2,])))*100) %>% print()
        
        print("confusion matrix across CV")
        caret::confusionMatrix(matrix) %>% print()
        
        print("final model confusion matrix")
        caret::confusionMatrix(predict(model, data), data$voter) %>% print()
        model_final <- model$finalModel
        print("variable importance")
        caret::varImp(model, scale = T) %>% print()
        caret::varImp(model, scale = F) %>% print()
        importance(model_final, scale = T) %>% print()
        model$finalModel$variable.importance %>% print()
}
#Note: In this function sometimes the same scores are calculated twice (or even more often) in different ways; e.g. applying different R-functions or different mathematical functions
#This is done in order to make sure everything works fine / results are correct!

#### Domain Level ####
## create data set only including necessary predictor variables and criterion variable ##
voter_domain <- final_data[,c("open", "consc", "extra", "agree", "neuro", "voter")]

## train model and test model ##
function_voter1(voter_domain)


#### Facet Level ####
## create data set only including necessary predictor variables and criterion variable ##
voter_facet <- final_data[,c("open_aes", "open_id", "consc_ord", "consc_sd", "extra_ass", "extra_act", "agree_altr", "agree_com","neuro_anx",  "neuro_dep", "voter")]

## train model and test model ##
function_voter1(voter_facet)


#### Nuance Level ####
## create data set only including necessary predictor variables and criterion variable ##
voter_nuance <- final_data[,c("BFI_01", "BFI_02_R", "BFI_03", "BFI_04", "BFI_05", "BFI_06_R", "BFI_07", "BFI_08_R", "BFI_09_R", "BFI_10",
                               "BFI_11", "BFI_12_R", "BFI_13", "BFI_14", "BFI_15", "BFI_16", "BFI_17", "BFI_18_R", "BFI_19", "BFI_20",
                               "BFI_21_R", "BFI_22", "BFI_23_R", "BFI_24_R", "BFI_25", "BFI_26", "BFI_27_R", "BFI_28", "BFI_29", "BFI_30",
                               "BFI_31_R", "BFI_32", "BFI_33", "BFI_34_R", "BFI_35_R", "BFI_36", "BFI_37_R", "BFI_38", "BFI_39", "BFI_40",
                               "BFI_41_R", "BFI_42", "BFI_43_R", "BFI_44", "voter")]

## train model and test model ##
function_voter1(voter_nuance)


####################################################
## Predict voting intentions for a specific party ## 
####################################################
final_data_voter <- final_data %>% filter(voter == "x1") 

#### The folds need to be the same across models in 10 x 10 CV to predict voting intentions for specific party: ####
set.seed(seed)
multifolds_partypref <- createMultiFolds(final_data_voter$partypref2, k = 10, times = 10)
#creates 10x10 folds with similar distribution of partypref2 variable in all folds

#### trainControl is similar across all analyses: ####
trcontr_w2 <- trainControl(method = "repeatedcv", number = 10, repeats = 10, returnData = T, returnResamp = "all", savePredictions = "all", classProbs = T, index = multifolds_partypref) # 10-fold cross validation and class probabilities and predictions should be saved; Multifolds (see above) are used

##### writing the function #####
table(final_data_voter$partypref2) # to calculate weights (see main manuscript for formula)
prop.table(table(final_data_voter$partypref2))

function_partypref2 = function(data){ # write one function to combine all necessary steps for the analyses. You can use the same function to predict voting intentions / party preference by Big Five personality domains, facets, and items
        grid = expand.grid(mtry = round(sqrt(ncol(data)-1)), splitrule = "gini", min.node.size = 1) # set variables sampled at each split to square root of number of predictor variables, splitting rule to "gini", and minimal node size to 1
        weights_voter <- ifelse(data$partypref2 == "x1", 1769/412,
                                ifelse(data$partypref2 == "x2", 1769/331,
                                       ifelse(data$partypref2 == "x3", 1,
                                              ifelse(data$partypref2 == "x4", 1769/292,
                                                     ifelse(data$partypref2 == "x5", 1769/586,
                                                            ifelse(data$partypref2 == "x6", 1769/187, 1769/464))))))
        set.seed(seed) 
        model <- train(partypref2 ~ ., data = data, # model training (predict "partypref2" from other variables in data set specified as "data")
                       method = "ranger", # implements random forest analyses using the ranger method/package
                       weights = weights_voter, # set weights (see above)
                       trControl = trcontr_w2, # set trainControl (see above)
                       metric = "Kappa", # Kappa is used because it is better than accuracy for unbalanced prediction problems; but not needed here because all tuning parameters are set
                       tuneGrid = grid, # set tuneGrid (see above)
                       importance = "permutation") # calculate permutation importance
        print("model")
        model %>% print()
        print("cross-validation results")
        model$results %>% print()
        model$resample %>% print()
        print("final model")
        model$finalModel %>% print ()
        print("confusion Matrix across repeated CV")
        matrix <- confusionMatrix.train(model, norm = "none")$table %>% print()
        print("Classification errors")
        (((matrix[2,1] + matrix[3,1] + matrix[4,1] + matrix[5,1] + matrix[6,1] + matrix[7,1])/(matrix[1,1] + matrix[2,1] + matrix[3,1] + matrix[4,1] + matrix[5,1] + matrix[6,1] + matrix[7,1]))*100) %>% print()
        (((matrix[1,2] + matrix[3,2] + matrix[4,2] + matrix[5,2] + matrix[6,2] + matrix[7,2])/(matrix[1,2] + matrix[2,2] + matrix[3,2] + matrix[4,2] + matrix[5,2] + matrix[6,2] + matrix[7,2]))*100) %>% print()
        (((matrix[1,3] + matrix[2,3] + matrix[4,3] + matrix[5,3] + matrix[6,3] + matrix[7,3])/(matrix[1,3] + matrix[2,3] + matrix[3,3] + matrix[4,3] + matrix[5,3] + matrix[6,3] + matrix[7,3]))*100) %>% print()
        (((matrix[1,4] + matrix[2,4] + matrix[3,4] + matrix[5,4] + matrix[6,4] + matrix[7,4])/(matrix[1,4] + matrix[2,4] + matrix[3,4] + matrix[4,4] + matrix[5,4] + matrix[6,4] + matrix[7,4]))*100) %>% print()
        (((matrix[1,5] + matrix[2,5] + matrix[3,5] + matrix[4,5] + matrix[6,5] + matrix[7,5])/(matrix[1,5] + matrix[2,5] + matrix[3,5] + matrix[4,5] + matrix[5,5] + matrix[6,5] + matrix[7,5]))*100) %>% print()
        (((matrix[1,6] + matrix[2,6] + matrix[3,6] + matrix[4,6] + matrix[5,6] + matrix[7,6])/(matrix[1,6] + matrix[2,6] + matrix[3,6] + matrix[4,6] + matrix[5,6] + matrix[6,6] + matrix[7,6]))*100) %>% print()
        (((matrix[1,7] + matrix[2,7] + matrix[3,7] + matrix[4,7] + matrix[5,7] + matrix[6,7])/(matrix[1,7] + matrix[2,7] + matrix[3,7] + matrix[4,7] + matrix[5,7] + matrix[6,7] + matrix[7,7]))*100) %>% print()
        print("sensitivity = recall")
        rec1 <- (matrix[1,1]/(matrix[1,1] + matrix[2,1] + matrix[3,1] + matrix[4,1] + matrix[5,1] + matrix[6,1] + matrix[7,1])) %>% print()
        rec2 <- (matrix[2,2]/(matrix[1,2] + matrix[2,2] + matrix[3,2] + matrix[4,2] + matrix[5,2] + matrix[6,2] + matrix[7,2])) %>% print()
        rec3 <- (matrix[3,3]/(matrix[1,3] + matrix[2,3] + matrix[3,3] + matrix[4,3] + matrix[5,3] + matrix[6,3] + matrix[7,3])) %>% print()
        rec4 <- (matrix[4,4]/(matrix[1,4] + matrix[2,4] + matrix[3,4] + matrix[4,4] + matrix[5,4] + matrix[6,4] + matrix[7,4])) %>% print()
        rec5 <- (matrix[5,5]/(matrix[1,5] + matrix[2,5] + matrix[3,5] + matrix[4,5] + matrix[5,5] + matrix[6,5] + matrix[7,5])) %>% print()
        rec6 <- (matrix[6,6]/(matrix[1,6] + matrix[2,6] + matrix[3,6] + matrix[4,6] + matrix[5,6] + matrix[6,6] + matrix[7,6])) %>% print()
        rec7 <- (matrix[7,7]/(matrix[1,7] + matrix[2,7] + matrix[3,7] + matrix[4,7] + matrix[5,7] + matrix[6,7] + matrix[7,7])) %>% print()
        
        print("specificity - confusion Matrix")
        caret::confusionMatrix(matrix) %>% print()
        
        print("precision")
        prec1 <- (matrix[1,1]/(matrix[1,1] + matrix[1,2] + matrix[1,3] + matrix[1,4] + matrix[1,5] + matrix[1,6] + matrix[1,7])) %>% print()
        prec2 <- (matrix[2,2]/(matrix[2,1] + matrix[2,2] + matrix[2,3] + matrix[2,4] + matrix[2,5] + matrix[2,6] + matrix[2,7])) %>% print()
        prec3 <- (matrix[3,3]/(matrix[3,1] + matrix[3,2] + matrix[3,3] + matrix[3,4] + matrix[3,5] + matrix[3,6] + matrix[3,7])) %>% print()
        prec4 <- (matrix[4,4]/(matrix[4,1] + matrix[4,2] + matrix[4,3] + matrix[4,4] + matrix[4,5] + matrix[4,6] + matrix[4,7])) %>% print()
        prec5 <- (matrix[5,5]/(matrix[5,1] + matrix[5,2] + matrix[5,3] + matrix[5,4] + matrix[5,5] + matrix[5,6] + matrix[5,7])) %>% print()
        prec6 <- (matrix[6,6]/(matrix[6,1] + matrix[6,2] + matrix[6,3] + matrix[6,4] + matrix[6,5] + matrix[6,6] + matrix[6,7])) %>% print()
        prec7 <- (matrix[7,7]/(matrix[7,1] + matrix[7,2] + matrix[7,3] + matrix[7,4] + matrix[7,5] + matrix[7,6] + matrix[7,7])) %>% print()

        print("F1 score")
        F1_1 <- ((2*(prec1*rec1)) / (prec1+rec1)) %>% print()
        F1_1_2 <- (2 / ((1/rec1)+(1/prec1))) %>% print() # different formulas are used in order to make sure no error occurs
        F1_2 <- ((2*(prec2*rec2)) / (prec2+rec2)) %>% print()
        F1_3 <- ((2*(prec3*rec3)) / (prec3+rec3)) %>% print()
        F1_4 <- ((2*(prec4*rec4)) / (prec4+rec4)) %>% print()
        F1_5 <- ((2*(prec5*rec5)) / (prec5+rec5)) %>% print()
        F1_6 <- ((2*(prec6*rec6)) / (prec6+rec6)) %>% print()
        F1_7 <- ((2*(prec7*rec7)) / (prec7+rec7)) %>% print()
        
        print("accuracy of model compared to no information rate")
        (((((matrix[1,1] + matrix[2,2] + matrix[3,3] + matrix[4,4] + matrix[5,5] + matrix[6,6] + matrix[7,7]) / 40410) - (as.matrix(prop.table(table(data$partypref2)))[3,])))*100) %>% print()
        
        print("final model confusion matrix")
        caret::confusionMatrix(predict(model, data), data$partypref2) %>% print()
        model_final <- model$finalModel
        print("variable importance")
        caret::varImp(model, scale = T) %>% print()
        caret::varImp(model, scale = F) %>% print()
        importance(model_final, scale = T) %>% print()
        model$finalModel$variable.importance %>% print()
}
#Note: In this function sometimes the same scores are calculated twice (or even more often) in different ways; e.g. applying different R-functions or different mathematical functions
#This is done in order to make sure everything works fine / results are correct!


#### Domain Level ####
## create data set only including necessary predictor variables and criterion variable ##
partypref_domain <- final_data_voter[,c("open", "consc", "extra", "agree", "neuro", "partypref2")]

## train model and test model ##
function_partypref2(partypref_domain)


#### Facet Level ####
## create data sets only including necessary predictor variables and criterion variable ##
partypref_facet <- final_data_voter[,c("open_aes", "open_id", "consc_ord", "consc_sd", "extra_ass", "extra_act", "agree_altr", "agree_com", "neuro_anx", "neuro_dep", "partypref2")]

## train model and test model ##
function_partypref2(partypref_facet)



#### Nuance Level ####
## create data sets only including necessary predictor variables and criterion variable ##
partypref_nuance <- final_data_voter[,c("BFI_01", "BFI_02_R", "BFI_03", "BFI_04", "BFI_05", "BFI_06_R", "BFI_07", "BFI_08_R", "BFI_09_R", "BFI_10",
                                        "BFI_11", "BFI_12_R", "BFI_13", "BFI_14", "BFI_15", "BFI_16", "BFI_17", "BFI_18_R", "BFI_19", "BFI_20",
                                        "BFI_21_R", "BFI_22", "BFI_23_R", "BFI_24_R", "BFI_25", "BFI_26", "BFI_27_R", "BFI_28", "BFI_29", "BFI_30",
                                        "BFI_31_R", "BFI_32", "BFI_33", "BFI_34_R", "BFI_35_R", "BFI_36", "BFI_37_R", "BFI_38", "BFI_39", "BFI_40",
                                        "BFI_41_R", "BFI_42", "BFI_43_R", "BFI_44", "partypref2")]

## train model and test model ##
function_partypref2(partypref_nuance)




########################################################
# Predict voting intentions for left- vs right parties # 
########################################################
final_data_lr <- final_data %>% filter(partypref != 7 & partypref != 8) 


#### The folds need to be the same across models in 10 x 10 CV to predict left- vs. right party voting intentions: ####
set.seed(seed)
multifolds_lr <- createMultiFolds(final_data_lr$partypref2leftright, k = 10, times = 10)
#creates 10x10 folds with similar distribution of partypref2leftright variable in all folds

#### trainControl is similar across all analyses: ####
trcontr_w3 <- trainControl(method = "repeatedcv", number = 10, repeats = 10, returnData = T, returnResamp = "all", savePredictions = "all", classProbs = T, index = multifolds_lr) # 10-fold cross validation and class probabilities and predictions are saved, Multifolds (see above) are used

##### writing the function #####
table(final_data_lr$partypref2leftright) # to calculate weights (see main manuscript for formula)
prop.table(table(final_data_lr$partypref2leftright))

function_lr3 = function(data){ # write one function to combine all necessary steps for the analyses. You can use the same function to predict non-voting vs voting by Big Five personality domains, facets, and items
        grid = expand.grid(mtry = round(sqrt(ncol(data)-1)), splitrule = "gini", min.node.size = 1) # set variables sampled at each split to square root of number of predictor variables, splitting rule to "gini", and minimal node size to 1
        weights_voter <- ifelse(data$partypref2leftright == "x0", 1, 2512/1065) # set weights
        set.seed(seed) 
        model <- train(partypref2leftright ~ ., data = data, # model training (predict "partypref2leftright" from other variables in data set specified as "data")
                       method = "ranger", # implements random forest analyses using the ranger method/package
                       weights = weights_voter, # set weights (see above)
                       trControl = trcontr_w3, # set trainControl (see above)
                       metric = "Kappa", # Kappa is used because it is better than accuracy for unbalanced prediction problems; but not needed here because all tuning parameters are set
                       tuneGrid = grid, # set tuneGrid (see above)
                       importance = "permutation") # calculate permutation importance
        print("model")
        model %>% print()
        print("cross-validation results")
        model$results %>% print()
        model$resample %>% print()
        print("final model")
        model$finalModel %>% print ()
        print("confusion Matrix across repeated CV")
        matrix <- confusionMatrix.train(model, norm = "none")$table %>% print()
        print("Classification errors")
        ((matrix[2,1]/(matrix[1,1] + matrix[2,1]))*100) %>% print()
        ((matrix[1,2]/(matrix[1,2] + matrix[2,2]))*100) %>% print()
        print("sensitivity = recall")
        rec <- (matrix[1,1]/(matrix[1,1] + matrix[2,1])) %>% print()
        print("specificity")
        (matrix[2,2]/(matrix[1,2] + matrix[2,2])) %>% print()
        print("precision")
        prec <- (matrix[1,1]/(matrix[1,1] + matrix[1,2])) %>% print()
        print("F1 score")
        F1_1 <- ((2*(prec*rec)) / (prec+rec)) %>% print()
        F1_2 <- (2 / ((1/rec)+(1/prec))) %>% print()
        F1_3 <- (matrix[1,1] / (matrix[1,1] + 0.5*(matrix[2,1] + matrix[1,2]))) %>% print() # different formulas are used in order to make sure no error occurs
        print("accuracy of model compared to no information rate")
        (((((matrix[1,1] + matrix[2,2]) / (matrix[1,1] + matrix[1,2] + matrix[2,1] + matrix[2,2])) - (as.matrix(prop.table(table(data$partypref2leftright)))[1,])))*100) %>% print()
        
        print("confusion matrix across CV")
        caret::confusionMatrix(matrix) %>% print()
        
        print("final model confusion matrix")
        caret::confusionMatrix(predict(model, data), data$partypref2leftright) %>% print()
        model_final <- model$finalModel
        print("variable importance")
        caret::varImp(model, scale = T) %>% print()
        caret::varImp(model, scale = F) %>% print()
        importance(model_final, scale = T) %>% print()
        model$finalModel$variable.importance %>% print()
        
}
#Note: In this function sometimes the same scores are calculated twice (or even more often) in different ways; e.g. applying different R-functions or different mathematical functions
#This is done in order to make sure everything works fine / results are correct!


###Domain Level###
## create data sets only including necessary predictor variables and criterion variable ##
lr_domain <- final_data_lr[,c("open", "consc", "extra", "agree", "neuro", "partypref2leftright")]

## train model and test model ##
function_lr3(lr_domain)

#### Facet Level ####
## create data sets only including necessary predictor variables and criterion variable ##
lr_facet <- final_data_lr[,c("open_aes", "open_id", "consc_ord", "consc_sd", "extra_ass", "extra_act", "agree_altr", "agree_com", "neuro_anx", "neuro_dep", "partypref2leftright")]

## train model and test model ##
function_lr3(lr_facet)


#### Nuance Level ####
## create data sets only including necessary predictor variables and criterion variable ##
lr_nuance <- final_data_lr[,c("BFI_01", "BFI_02_R", "BFI_03", "BFI_04", "BFI_05", "BFI_06_R", "BFI_07", "BFI_08_R", "BFI_09_R", "BFI_10",
                               "BFI_11", "BFI_12_R", "BFI_13", "BFI_14", "BFI_15", "BFI_16", "BFI_17", "BFI_18_R", "BFI_19", "BFI_20",
                               "BFI_21_R", "BFI_22", "BFI_23_R", "BFI_24_R", "BFI_25", "BFI_26", "BFI_27_R", "BFI_28", "BFI_29", "BFI_30",
                               "BFI_31_R", "BFI_32", "BFI_33", "BFI_34_R", "BFI_35_R", "BFI_36", "BFI_37_R", "BFI_38", "BFI_39", "BFI_40",
                               "BFI_41_R", "BFI_42", "BFI_43_R", "BFI_44", "partypref2leftright")]

## train model and test model ##
function_lr3(lr_nuance)




#######################################
#### Logistic Regression Analyses #####
#######################################

#### predicting dependent variable by means of logistic regression models for easier interpretation; results are presented in Supplementary Material ####
library(car) # to check VIF (multicollinearity in models)

################################################
######## Predict non-voting vs. voting ######### 
################################################


#### Domain level ####
log_voter_domain <- glm(voter ~  ., data=voter_domain, family=binomial(link=logit)) # run binary logistic regression model

cooksd_dom1 <- cooks.distance(log_voter_domain) # calculating cook's distance to check for outliers
plot(cooksd_dom1) # plot cook's distance scores
abline(h = 4/nrow(voter_domain), col="red") # threshold 1
abline(h = 4*mean(cooksd_dom1, na.rm=T), col="blue") # threshold 2
par(mfrow=c(2,2)); plot(log_voter_domain); par(mfrow=c(1,1)) # there are outliers
car::vif(log_voter_domain) # VIF scores seem to be ok (no multicollinearity assumed); but note various different rules of thumb on how to interpret VIF scores

summary(log_voter_domain) # present results
exp(coef(log_voter_domain)) # exponents of estimates


#### Facet level ####
log_voter_facet <- glm(voter ~  ., data=voter_facet, family=binomial(link=logit))

cooksd_fac1 <- cooks.distance(log_voter_facet) # calculating cook's distance to check for outliers
par(mfrow=c(1,1)); plot(cooksd_fac1) # plot cook's distance scores
abline(h = 4/nrow(voter_facet), col="red") # threshold 1
abline(h = 4*mean(cooksd_nuan2, na.rm=T), col="blue") # threshold 2
par(mfrow=c(2,2)); plot(log_voter_facet) # there are outliers
car::vif(log_voter_facet) # VIF scores seem to be ok (no multicollinearity assumed); but note various different rules of thumb on how to interpret VIF scores

summary(log_voter_facet)
exp(coef(log_voter_facet))


#### Nuance level ####
log_voter_nuance <- glm(voter ~  ., data=voter_nuance, family=binomial(link=logit))

cooksd_nuan1 <- cooks.distance(log_voter_nuance) # calculating cook's distance to check for outliers
par(mfrow=c(1,1)); plot(cooksd_nuan1) # plot cook's distance scores
abline(h = 4/nrow(voter_nuance), col="red") # threshold 1
abline(h = 4*mean(cooksd_nuan1, na.rm=T), col="blue") # threshold 2
par(mfrow=c(2,2)); plot(log_voter_nuance) # there are outliers
car::vif(log_voter_nuance) # VIF scores seem to be ok (no multicollinearity assumed); but note various different rules of thumb on how to interpret VIF scores

summary(log_voter_nuance)
exp(coef(log_voter_nuance))



##################################################
# Predict voting for a specific party (alliance) #
##################################################

#### Domain level ####
final_data_voter$linke <- ifelse(final_data_voter$partypref2 == "x1", 1, 0)
final_data_voter$spd <- ifelse(final_data_voter$partypref2 == "x2", 1, 0)
final_data_voter$grün <- ifelse(final_data_voter$partypref2 == "x3", 1, 0)
final_data_voter$fdp <- ifelse(final_data_voter$partypref2 == "x4", 1, 0)
final_data_voter$cducsu <- ifelse(final_data_voter$partypref2 == "x5", 1, 0)
final_data_voter$afd <- ifelse(final_data_voter$partypref2 == "x6", 1, 0)
final_data_voter$other <- ifelse(final_data_voter$partypref2 == "x7", 1, 0)

function_log1 = function(data, y) {
        print("domain")
        model_domain <- glm(data[[y]] ~  open + consc + extra + agree + neuro, data=data, family=binomial(link=logit)) # logistic regression domain level
        cooksd_dom <- cooks.distance(model_domain)
        par(mfrow=c(1,1)); plot(cooksd_dom) # plot cook's distance scores
        abline(h = 4/nrow(data), col="red") # threshold 1
        abline(h = 4*mean(cooksd_dom, na.rm=T), col="blue") # threshold 2
        car::vif(model_domain) %>% print()
        summary(model_domain) %>% print()
        exp(coef(model_domain)) %>% print()
        
        print("facet")
        model_facet <- glm(data[[y]] ~ open_aes + open_id + consc_ord + consc_sd + extra_ass + extra_act + agree_altr + agree_com + neuro_anx + neuro_dep, data=data, family=binomial(link=logit)) # logistic regression facet level
        cooksd_fac <- cooks.distance(model_facet)
        par(mfrow=c(1,1)); plot(cooksd_fac) # plot cook's distance scores
        abline(h = 4/nrow(data), col="red") # threshold 1
        abline(h = 4*mean(cooksd_fac, na.rm=T), col="blue") # threshold 2
        car::vif(model_facet) %>% print()
        summary(model_facet) %>% print()
        exp(coef(model_facet)) %>% print()
        
        print("nuance")
        model_nuance <- glm(data[[y]] ~ BFI_01 + BFI_02_R + BFI_03 + BFI_04 + BFI_05 + BFI_06_R + BFI_07 + BFI_08_R + BFI_09_R + BFI_10 +
                               BFI_11 + BFI_12_R + BFI_13 + BFI_14 + BFI_15 + BFI_16 + BFI_17 + BFI_18_R + BFI_19 + BFI_20 + 
                               BFI_21_R + BFI_22 + BFI_23_R + BFI_24_R + BFI_25 + BFI_26 + BFI_27_R + BFI_28 + BFI_29 + BFI_30 +
                               BFI_31_R + BFI_32 + BFI_33 + BFI_34_R + BFI_35_R + BFI_36 + BFI_37_R + BFI_38 + BFI_39 + BFI_40 +
                               BFI_41_R + BFI_42 + BFI_43_R + BFI_44, data=data, family=binomial(link=logit)) # logistic regression item level
        cooksd_nuan <- cooks.distance(model_nuance)
        par(mfrow=c(1,1)); plot(cooksd_nuan) # plot cook's distance scores
        abline(h = 4/nrow(data), col="red") # threshold 1
        abline(h = 4*mean(cooksd_nuan, na.rm=T), col="blue") # threshold 2
        car::vif(model_nuance) %>% print()
        summary(model_nuance) %>% print()
        exp(coef(model_nuance)) %>% print()
        
} 

### run function (see above); implements one-vs-all approach for predictions of intentions to vote for each party; ###
### implements logistic regression on domain, facet, item level ###
function_log1(final_data_voter, "linke") 
function_log1(final_data_voter, "spd")
function_log1(final_data_voter, "grün")
function_log1(final_data_voter, "fdp")
function_log1(final_data_voter, "cducsu")
function_log1(final_data_voter, "afd")
function_log1(final_data_voter, "other")



####################################################
# Predict voting for left-/right parties/alliances #
####################################################

#### Domain level ####
log_lr_domain <- glm(partypref2leftright ~  ., data=lr_domain, family=binomial(link=logit))

cooksd_dom3 <- cooks.distance(log_lr_domain) # calculating cook's distance to check for outliers
par(mfrow=c(1,1)); plot(cooksd_dom3) # plot cook's distance scores
abline(h = 4/nrow(lr_domain), col="red") # threshold 1
abline(h = 4*mean(cooksd_dom3, na.rm=T), col="blue") # threshold 2
par(mfrow=c(2,2)); plot(log_lr_domain) # there are outliers
car::vif(log_lr_domain) # VIF scores seem to be ok (no multicollinearity assumed); but note various different rules of thumb on how to interpret VIF scores

summary(log_lr_domain)
exp(coef(log_lr_domain))


#### Facet level ####
log_lr_facet <- glm(partypref2leftright ~  ., data=lr_facet, family=binomial(link=logit))

cooksd_fac3 <- cooks.distance(log_lr_facet) # calculating cook's distance to check for outliers
par(mfrow=c(1,1)); plot(cooksd_fac3) # plot cook's distance scores
abline(h = 4/nrow(lr_facet), col="red") # threshold 1
abline(h = 4*mean(cooksd_fac3, na.rm=T), col="blue") # threshold 2
par(mfrow=c(2,2)); plot(log_lr_facet) # there are outliers
car::vif(log_lr_facet) # VIF scores seem to be ok (no multicollinearity assumed); but note various different rules of thumb on how to interpret VIF scores

summary(log_lr_facet)
exp(coef(log_lr_facet))


#### Nuance level ####
log_lr_nuance <- glm(partypref2leftright ~  ., data=lr_nuance, family=binomial(link=logit))

cooksd_nuan3 <- cooks.distance(log_lr_nuance) # calculating cook's distance to check for outliers
par(mfrow=c(1,1)); plot(cooksd_nuan3) # plot cook's distance scores
abline(h = 4/nrow(lr_nuance), col="red") # threshold 1
abline(h = 4*mean(cooksd_nuan3, na.rm=T), col="blue") # threshold 2
par(mfrow=c(2,2)); plot(log_lr_nuance) # there are outliers
car::vif(log_lr_nuance) # VIF scores seem to be ok (no multicollinearity assumed); but note various different rules of thumb on how to interpret VIF scores

summary(log_lr_nuance)
exp(coef(log_lr_nuance))


#######################################
############### THE END ############### 
#######################################